from enum import Enum

class Lookback(Enum):
    ONE_HOUR = "1H"
    SIX_HOURS = "6H"
    TWELVE_HOURS = "12H"
    ONE_DAY = "1D"
    ONE_WEEK = "7D"
    ONE_MONTH = "30D"
    THREE_MONTHS = "90D"
    ONE_YEAR = "365D"


class EntityType(Enum):
    FINANCIAL_ASSET = "financial_asset"
    COMPANY = "company"
    ORGANIZATION = "organization"
    PERSON = "person"
    PLACE = "place"


class AssetClass(Enum):
    CRYPTO = "crypto"
    COMMODITIES = "commodities"


class EntityMetrics(Enum):
    MENTIONS = 'mentions'
    SENTIMENT = 'sentiment'
    EXCITEMENT = 'excitement'
    OPTIMISM = 'optimism'
    PESSIMISM = 'pessimism'
    FEAR = 'fear'
    UNCERTAINTY = 'uncertainty'
    SURPRISE = 'surprise'



import datetime as dt
import os
import unittest

from rumorz.client import RumorzClient

rumorz = RumorzClient(api_key=os.environ['RUMORZ_API_KEY'])
                      #api_url='http://localhost:8000')


class TestRumorz(unittest.TestCase):


    def test_search_entities(self):
        entities = rumorz.graph.search_entities(**{
            "name": "Bitcoin",
            "asset_class": "crypto",
            "entity_type": "FinancialAsset",
            "symbol_search": "BTC",
            "limit": 10
        })
        self.assertTrue(len(entities) > 0)

    def test_time_series(self):
        timeseries = rumorz.graph.get_entity_timeseries(**{
            "node_ids": [
                "7d8d81b3-0808-47ce-b459-a9fd5f74fd57"
            ],
            "metrics": [
                "price",
                "sentiment"
            ],
            "lookback": "3M",
            "page": 1,
            "limit": 100
        })
        self.assertTrue(len(timeseries) > 0)

    def test_entity_summary(self):
        entity_update = rumorz.agent.get_entity_summary(**{
            "node_id": "7d8d81b3-0808-47ce-b459-a9fd5f74fd57",
            "timestamp": dt.datetime.utcnow().isoformat(),
            "scores_filter": "sentiment > 0.75"
        })
        self.assertTrue(len(entity_update) > 0)

    def test_get_ranking(self):
        screener = rumorz.graph.get_ranking(**{
            "lookback": "1D"
        })
        self.assertTrue(len(screener) > 0)


    def test_posts(self):
        posts = rumorz.graph.get_entity_posts(**{
            "node_ids": ["7d8d81b3-0808-47ce-b459-a9fd5f74fd57"],
            "lookback": "30D",
            "scores_filter": "sentiment > 0.75",
            "page": 1,
            "limit": 10
        })

        self.assertTrue(len(posts) > 0)



import importlib.util
import json
import subprocess
import traceback
from textwrap import dedent

from litellm import completion
from pydantic import BaseModel, Field


class RumorzCopilot:

    def __init__(self,
                 model):
        self.model = model

    def load_sdk_source_code(self):
        package_name = "rumorz"
        spec = importlib.util.find_spec(package_name)
        # Find the spec for the package
        if spec and spec.origin:
            sdk_path = spec.origin.replace('__init__.py', '')
        else:
            raise Exception(f"Package {package_name} not found  ")

        client_path = sdk_path + 'client.py'
        enums_path = sdk_path + 'enums.py'
        test_path = sdk_path + 'tests/test_sdk.py'
        # read content
        with open(client_path, 'r') as file:
            client_content = file.read()
        with open(enums_path, 'r') as file:
            enums_content = file.read()
        with open(test_path, 'r') as file:
            test_content = file.read()
        docs = dedent(f"""
<RUMORZ SDK SOURCE CODE>
file_path: {client_path}
{client_content}
<ENUMS>
file_path: {enums_path}
{enums_content}
<TESTS>
file_path: {test_path}
{test_content}
""")

        return docs

    def run_script(self, script_path):
        result = subprocess.run(['python', script_path], capture_output=True, text=True)
        return result.stdout

    def generate_script(self, script_description, output_path):

        source_code = self.load_sdk_source_code()

        class SDKCopilotResponse(BaseModel):
            reasoning: str = Field(...,
                                   description="Write a complete reasoning and planning on requirements and steps to write the script based on the documentation and the provided script description")
            script_code: str = Field(..., description="The Python script code")

        response = completion(
            model=self.model,
            response_format=SDKCopilotResponse,
            messages=[
                {"role": "system", "content": f"""
ROLE:
You are an expert in Programming and the Rumorz Python SDK. Rumorz provides Financial markets AI intelligence through
the Graph and an Agent. The Graph gives access to entities (tokens, people, companies, etc.) and their posts, sentiment
etc. The Agent provides summaries and insights on entities. 

TASK
You are tasked to create a fully working Python script based on user input. The script must use the Rumorz Python SDK. 
Your output will be in json

GUIDELINES
- Use pandas sort_values to rank by the desired metric
- Minimize the number of API queries.
- If you hesitate, tell the user to ask again with specific improvements to make in his query
- Your script should always print a success message with details at the end of the execution

DOCUMENTATION
Use the following documentation and Enums to build working queries:
{source_code}
"""},
                {"role": "user",
                 "content": f"The user message to describe the script required: {script_description}"},
            ],

        )

        class RumorzCopilotException(Exception):
            pass

        try:
            script_code = json.loads(response.choices[0].message.content)['script_code']
            with open(output_path, 'w') as f:
                f.write(script_code)
        except Exception as e:
            raise RumorzCopilotException("Error while parsing LLM response ", traceback.format_exc())



import os
import traceback
from enum import Enum

import litellm
import pandas as pd
import requests

litellm.set_verbose = False


class RumorzAPIException(Exception):
    pass


class RumorzClient:
    def __init__(self,
                 copilot_model='azure/gpt-4o-mini',
                 api_key=os.environ['RUMORZ_API_KEY'],
                 api_url='http://rumorz-api.eastus2.azurecontainer.io'):
        self.api_url = api_url
        self.headers = {
            'accept': 'application/json',
            'Content-Type': 'application/json',
            'X-API-Key': api_key
        }
        self.graph = self.Graph(self)
        self.agent = self.Agent(self,
                                copilot_model)

    def call_function(self, function_name, params):
        url = f"{self.api_url}/api/functions"
        params = self._format_params(params)
        payload = {
            "function": function_name,
            "params": params
        }
        response = requests.post(url, json=payload, headers=self.headers, timeout=30)
        try:
            response.raise_for_status()
            return response.json()['data']
        except requests.exceptions.HTTPError as e:
            if response.status_code == 500:
                raise RumorzAPIException(response.json()['detail'])
            else:
                raise RumorzAPIException(traceback.format_exc())

    def _format_params(self, data):
        for key, value in data.items():
            if isinstance(value, Enum):
                data[key] = value.value

        todrop = []
        for key, value in data.items():
            if value is None:
                todrop.append(key)
        for key in todrop: data.pop(key)
        return data

    class Graph:

        def __init__(self, api):
            self.api = api

        def get_entity_posts(self,
                             **kwargs):
            return self.api.call_function('get_entity_posts', params=kwargs)

        def search_entities(self,
                            **kwargs):
            """
            Search for entities in the database based on the given parameters.
            Example:
            entities = rumorz.graph.search_entities(name=entity_name, symbol_search=symbol, asset_class=AssetClass.CRYPTO.value, entity_type=EntityType.FINANCIAL_ASSET.value, limit=1)

            Returns a list of json entities like {id, type, name, symbol}
            """
            return self.api.call_function('search_entities', params=kwargs)

        def get_ranking(self,
                        **kwargs):
            """
            Returns a ranking of every single entity in the Graph based on the given parameters.
            """
            return self.api.call_function('get_ranking', params=kwargs)

        def get_entity_timeseries(self,
                                  as_df=True,
                                  **kwargs):
            timeseries = self.api.call_function('get_entity_timeseries', params=kwargs)
            if as_df:
                dfs = {}
                for node_ts in timeseries:
                    df = pd.DataFrame()
                    df['timestamp'] = [v[0] for v in node_ts['time_series'][0]['values']]
                    df['timestamp'] = pd.to_datetime(df['timestamp'])
                    df.set_index('timestamp', inplace=True)
                    for ts in node_ts['time_series']:
                        metric = ts['metric']
                        values = ts['values']
                        df[metric] = [v[1] for v in values]
                        dfs[node_ts['node_id']] = df
                return dfs


    class Agent:
        def __init__(self, api, copilot_model):
            self.api = api
            self.copilot_model = copilot_model

        def get_entity_summary(self,
                               **kwargs):
            return self.api.call_function('get_entity_summary', params=kwargs)



